
# q-expansion of newform 36.15.d.c, downloaded from the LMFDB on 24 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 36
weight = 15
poly_data = [982333440, 21572352, 325632, -1824, 88, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[-18919936, -319936, 1424, -100, -3, -1], 262144], [[-84863488, 1295680, -233648, 29484, -503, 3], 131072], [[-176479744, 266195904, 9968, -700, -21, -7], 262144], [[-141571584, -7012672, 158128, 5268, 967, -19], 65536], [[709930496, -58592704, 8059536, -9636, -1907, 47], 131072]]

hecke_ring_character_values = [[19, [-1, 0, 0, 0, 0, 0]], [29, [1, 0, 0, 0, 0, 0]]]
aps_data = [[15, -1, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0], [-1363, -62, 0, 0, -3, 1], [-586, -1808, 14, 4, -40, -8], [34533, 102162, 277, -440, -720, -144], [-13330889, 395798, 0, 1664, -897, 299], [19693924, -997740, 0, -3136, -10494, 3498], [552481, 1692842, -29423, -5784, 11760, 2352], [-5983282, -17732192, -186074, 63620, -32040, -6408], [1324329461, -45071342, 0, -182784, 21381, -7127], [-34347816, -105456432, 3226008, 452384, 360640, 72128], [5218198335, -10269914, 0, -243584, 2437863, -812621], [-13651689734, 730386792, 0, 2829440, 1251108, -417036], [335527043, 1023507094, -20399389, -4007024, 533600, 106720], [-133766740, -421803728, -2273220, -103208, -22673520, -4534704], [-14179007655, -4185562774, 0, -17907584, 13230273, -4410091], [-891340799, -2609061214, -182232639, -190816, -117080640, -23416128], [1032026495719, -8969021962, 0, -34925440, -13189713, 4396571], [7615293013, 22088719826, 475817413, -94307800, -187034000, -37406800], [2593512058, 7450987040, 268543810, -28696404, -32308920, -6461784], [-4979827917204, 1972003948, 0, -3229376, 134328318, -44776106], [-2987871956, -10257378592, 1618595548, 70280584, 254552240, 50910448], [20720875647, 60375209598, 1493516671, -223455872, -70444800, -14088960], [30171664582948, 30627930036, 0, 257091776, -1615544622, 538514874], [-57706577018604, -393968566340, 0, -1383852224, -2389807866, 796602622], [-82063904434391, -396032177526, 0, -1346508160, -2939595759, 979865253], [73474598526, 236531400480, -13664997738, -764282716, 3206889880, 641377976], [319430045149, 965673073202, -3004613075, -3134703240, 7513027920, 1502605584], [165207130285995, -307401846674, 0, -1385939456, 1824052155, -608017385], [208652058236750, -615864695264, 0, -2490476544, 206320848, -68773616], [994041712512, 2896030558800, 58803373152, -11724456624, -15566748960, -3113349792], [557218803675, 1722007499934, -38563778101, -5548845592, 17336156400, 3467231280], [346309942112426, -3831021639032, 0, -16646702976, 15193906548, -5064635516], [-8507693141, 148568916230, -168794840885, -1352917072, 6650071840, 1330014368], [269265689381429, -393734998670, 0, -3044062464, 17624349429, -5874783143], [1832808382354, 5454919192352, -59241509318, -26848027268, -75899436760, -15179887352], [-490358674755857, -5922202835482, 0, -30337596544, 78960905031, -26320301677], [1923652321613, 5578019046538, 283431849965, -11456030736, 101949962400, 20389992480], [1251806682610, 3288120355088, 415457211066, -12290582356, -39551899320, -7910379864], [2986777840479665, -6683123722214, 0, -36151672704, 112192392633, -37397464211], [3541104808759, 11317386890846, -522773703209, -31866520080, 203165281440, 40633056288], [-4716710543657965, 19345929246462, 0, 64280801280, 161613373923, -53871124641], [-13894915913968, -40777261147472, -406900205712, 176132179600, 324454209120, 64890841824], [3900305297590468, 660009654236, 0, 50379070784, -575050595370, 191683531790], [4848186184895845, 29531883711954, 0, 136301237504, -213247956123, 71082652041], [-5892727811898, -18434851899504, 1002235731486, 89357543556, 156209724120, 31241944824], [-24686002163071, -72307934469806, -2035881206431, 221635923536, -507445110560, -101489022112], [-30331547721240, -88784121914064, -936548073432, 395898443104, 878074733120, 175614946624], [5136613953955, 11477028272966, 2752358400419, -103005071360, -1077450117120, -215490023424], [3719438731688107, 53924634168974, 0, 193552442624, 277261739787, -92420579929], [5525535876700900, 76649305131636, 0, 322438206656, -176022983502, 58674327834], [-20483912958404, -65350354920160, 4337727415980, 292400540072, 146710830960, 29342166192], [-23860888696579748, -200281139058580, 0, -831522320320, 327479937150, -109159979050], [-65271807790287, -195350197728126, -3652179058191, 444379167264, -3631332582720, -726266516544], [68276551030338398, -353348374272704, 0, -1546468978176, 1534954901280, -511651633760], [108209310357354, 333560310903168, -9114898211534, -1257492808148, 1074974427720, 214994885544], [-59797069730758787, -365574866806974, 0, -1380143820544, -1060599459267, 353533153089], [223778030792924, 664496355540496, 3596132638316, -2550741445800, -690862754160, -138172550832], [7573482974912055, -162967167748874, 0, -352879965824, -3633873689937, 1211291229979], [-26400134272636372, 613128324995620, 0, 2557616664000, -1147656016470, 382552005490], [215922089046359, 658377809672950, -12882780007033, -2578413390520, 307175917360, 61435183472], [65421348680566853, -141335356657262, 0, -593751531264, 314940707781, -104980235927], [-274910289188775, -813983518377030, -1294712547063, 3526772015112, 5925864627120, 1185172925424], [-104309550341578, -330779339707856, 16779841702414, 1214969647172, -2285816627880, -457163325576], [220169490947398438, 166342513006616, 0, 154009049984, 6193270623900, -2064423541300], [53989963911223877, -1811609825243918, 0, -6967731419136, -3708614607051, 1236204869017], [848768491808619, 2555256861435678, -19973289542757, -9947745045176, -1074158487760, -214831697552], [-5128058638092918, 1036614525942064, 0, 2657115548416, 18144810696120, -6048270232040], [584927034639913, 1668539600016866, 64892806055049, -6925649206384, -14423048641440, -2884609728288], [-16909500027470877, 1705941540811198, 0, 8704526412544, -22329945286077, 7443315095359], [123939574990577878, -419663027474352, 0, -3912370050304, 26831473308936, -8943824436312], [713826586230850, 2186549170169888, -59682263613782, -9173126678244, -5439725458200, -1087945091640], [-2190559832079676, -6622571139462416, 90918838889396, 26854005139048, 13173190526960, 2634638105392], [-744476593700842017, 4177149996629926, 0, 16034678260864, 8928150468135, -2976050156045], [-1260374473727217, -3731969183513490, -87846431753745, 9900599136336, -48592793221920, -9718558644384], [-1741412523499680, -5076308242741248, -121448385774880, 18939978535232, 7540963447680, 1508192689536], [528788965142966785, 1423720741079514, 0, 6270930448256, -6664884219495, 2221628073165], [1389850961656503003, 2206698796978318, 0, 7131283311616, 20855286405771, -6951762135257], [1322828731311608372, 4463939149593076, 0, 22692077714880, -57405880713486, 19135293571162], [74404365873252982, 7036611005078264, 0, 28993928422784, -8848895428788, 2949631809596], [1629931529883843, 4930834109592030, -13008543518125, -15596051163304, 43627027585680, 8725405517136], [-258085184630876045, -620131015062658, 0, -2166772295680, -3900228069597, 1300076023199], [1321647726355628, 3662561149769728, 340529993478428, -7904462366808, 46052426548080, 9210485309616], [-2900509615438552020, -3210353614860596, 0, -12961204262336, 821915151342, -273971717114], [5135192365485866, 15654147449182096, -176209185902254, -51621680648516, 123982847470760, 24796569494152], [-181215787018255, -464118202610390, -155650698032607, -4738730960312, -71382808627920, -14276561725584], [2653349292454079772, 7984731032407652, 0, 28808574575296, 39261094991562, -13087031663854], [1664856476960846710, -10047127830266824, 0, -32046659964544, -100040782214292, 33346927404764], [-2447489917074894227, -4062585297582750, 0, -13960000641280, -28381216350195, 9460405450065], [4771007298363852, 14371778420774880, -211271973584388, -62928609861304, -89586838039760, -17917367607952], [-1542439735122321, -5076808928520570, 457077482516703, 21725704094376, -14137944731280, -2827588946256], [6972080489749720, 20825286854910224, -154337955324136, -91796522048672, -153496047614400, -30699209522880], [1383645381615742, 3038114146171472, 970553031813814, -13749043995180, -128519922866760, -25703984573352], [1977575669941389, 5897645687012634, -294637442040691, -45822393355264, -283896371496960, -56779274299392], [4585478248286297, 13376985882857266, 245654750721881, -54876294984064, -78917816295680, -15783563259136], [-2364652639452474, -8106397995707424, 1272335210406654, 55714837312212, 204180295744440, 40836059148888], [-89818950491669807, 53121503565268506, 0, 208835517155456, 54264183559353, -18088061186451], [-8051751184551796422, -33408972746048920, 0, -137716033965440, 42692889258660, -14230963086220], [16617360754754299, 48778654800013598, 641956368240235, -197988590040024, -233482505969040, -46696501193808], [-3500902706509678953, 103574806661165590, 0, 416625129047680, -7976635727745, 2658878575915], [10930745226570141, 33143322939393186, 442327400136333, -59121517362584, 852536177181680, 170507235436336], [2567938622526379897, 3255571414241802, 0, 46276604127104, -400030790971311, 133343596990437], [-4860829280662425, -12717174909104634, -908150936235721, 104958415325240, 852203581321680, 170440716264336], [-4424671012827688774, 23542796750220328, 0, 55238216294016, 473636172242436, -157878724080812], [-42108820340693713, -125281867959805130, -509903025282625, 476794007906264, 57896029087120, 11579205817424], [-3981740849244807798, -6034378955697296, 0, -87817109613824, 766068451845720, -255356150615240], [-34628544266625547, -104799503852164622, 1851888525705381, 449003621311240, 489013852106160, 97802770421232], [-3998726729306489522, 137684406171392928, 0, 536568938686976, 197358408306576, -65786136102192], [8830184396013374, 23807607357257872, 3418798830643702, -9013768218988, 744866768080440, 148973353616088], [-20678879352474243812, 130923844035326924, 0, 519226244037440, 79185932419566, -26395310806522], [27768963180290920, 79606354984672160, 2498688108294472, -347364363577648, -854482084328480, -170896416865696], [16766272022407017395, 181861884247917054, 0, 705142413410304, 303939837520227, -101313279173409], [8792255533443050060, 146074943690606884, 0, 505686535449024, 975435307925802, -325145102641934], [-32290619026112829, -95922306663254178, 510401392570707, 444296407920856, 1015655399734160, 203131079946832], [-12833949881172070, -31918648735024544, -7399217539141534, 3985448322732, -820002078972600, -164000415794520], [-6355447562535799348, -153095587425608444, 0, -604786592228160, -121148030361126, 40382676787042], [49149759060511429, 145259756554733042, 684777272011189, -613528562507416, -891214792282640, -178242958456528], [-12045706565115958, -28140657405602432, -6733348181746414, 139529361496428, 1123584746502600, 224716949300520], [21332320461400976941, 21313833749710050, 0, 64330430855936, 256236603221709, -85412201073903], [-18444570878741809, -58331473226300354, 3130425049750671, 237765558157984, -105101098482240, -21020219696448], [39443182439908680643, 17548036149933854, 0, -8754511920640, 954576271847091, -318192090615697], [18199374706383099940, -172056119998888036, 0, -592668765889216, -1184593417377738, 394864472459246], [-65673116600386607, 32598110317674426, 0, 103550050903424, 329716821422025, -109905607140675], [122836404552898493, 364495297015119226, 751986486660029, -1510704724406784, -1751225432509440, -350245086501888], [131885844196625865, 398157797766726498, -3873487631977047, -1531563811575344, 158341356447200, 31668271289440], [84899796463256837737, 58513355049789546, 0, 363080894360960, -1543247789326911, 514415929775637], [25988998949943946495, -25250354739393754, 0, -322945782887296, 2669184445139175, -889728148379725], [-97756306363288524, -287696157533177856, -1068783458610876, 1322846996950680, 3181131101126160, 636226220225232], [131189874135500578, 388612904741083568, 7552238276400490, -1141670998630676, 3737191609613000, 747438321922600], [-49928875404588000933, -631216726048852658, 0, -2462365148865536, -875216321677461, 291738773892487], [-67941260651318691, -196971138249790470, -5743889867660835, 733244634500928, 375509202003840, 75101840400768], [-137258082131253566, -404321653221266128, -6265117135093110, 1468740089469740, -281264052068280, -56252810413656], [-23104422740249084, -54976118658073792, -15077806695836972, 36460104233528, -777117237397040, -155423447479408], [50724171263335272107, 357848807380728654, 0, 1186379840153856, 3021301315278315, -1007100438426105], [-76521538237283034214, 869702856200123752, 0, 3734343462188160, -2910419317002972, 970139772334324], [38408143192189232068, 389169875505231068, 0, 1783067507964224, -2652287127961002, 884095709320334], [70654315691879603569, -266466435414446598, 0, -1513706354233984, 5344182909184617, -1781394303061539], [-188809767402211151, -540457503862726702, -15545270516389295, 2525572923741296, 7900954903776160, 1580190980755232], [107230432280565115745, 820612635932391034, 0, 3286381622611584, 111465092037417, -37155030679139], [-8350336509461813222, 73569893460736296, 0, -271808478676864, 6834687321835908, -2278229107278636], [213231603230072787, 652747634667902238, -6253707160090077, -1882492355850120, 8681610173443920, 1736322034688784], [-19451761906838607871, 70992342569222554, 0, -26072891902080, 3749249644492281, -1249749881497427], [-350090716590318134, -1076210065368315376, 20864766950171506, 3604054797536252, -8677203444725720, -1735440688945144], [227765569268796401, 646376999977501114, 40666687053721217, -1739571242262312, 5486550467095440, 1097310093419088], [87706030427469603887, 1371489462135868134, 0, 5444068531914624, 770148275280327, -256716091760109], [292294875665347738, 854859226284072656, 2949850939555586, -4327106510772452, -14748443261642520, -2949688652328504], [248622787297391062935, 705123978982931254, 0, 2878810811484544, -566124826292913, 188708275430971], [5113459344344319770, -1698359921225180696, 0, -6587569095847296, -2809123372219548, 936374457406516], [-147493239189592681, -422975856665383106, -17591041994659081, 1504901002432016, 407917906303840, 81583581260768], [91833106839751928, 305968705434379120, -4932987941947912, 653498582292864, 24882898390882560, 4976579678176512], [203808196855620761059, -50387099101947778, 0, 1098815996024576, -15677045908648413, 5225681969549471], [-135007179733190222346, 938070978835114064, 0, 4579961161381120, -9790613726554104, 3263537908851368], [355392699617167465, 1071935401301221418, 3193201168827385, -3149749214579112, 12100252833125520, 2420050566625104], [-633332167219688698, -1840368995051900720, -45130586915876642, 7198327838532932, 7298591852755800, 1459718370551160], [97413581860696203, 249231612879405654, 18743509302123531, -2432239253100224, -21833384147459200, -4366676829491840], [873997470809023876, 2561768965324083536, 55840886218394036, -8973762867039416, 4591201982445360, 918240396489072], [340846244607978514, 981763417414778816, 59786124301105786, -1732410948798180, 20743218840747240, 4148643768149448], [-187309619866633120324, -3250904812297939292, 0, -13058487351768384, 31864756083306, -10621585361102], [165533646611544192060, -459036919238684084, 0, -3482865718618304, 19751450549868078, -6583816849956026], [330675976618938778853, 1457974881466564210, 0, 7247804152012800, -16777178341191435, 5592392780397145], [500651414086524039, 1386880801682934054, 96302387968751767, -5527914567799016, -13243138040087280, -2648627608017456], [-818076337983048769334, -890225216893338360, 0, -5516245230845824, 23386444141319604, -7795481380439868], [813330714385141830, 2373490877376100272, 51098479534476894, -9105408806462044, -6297377438386280, -1259475487677256], [554619174437957957, 1733358423892798762, -104339727971904763, -9288318813655392, -25550508579324480, -5110101715864896], [-660613506257837736284, 5738215181309993748, 0, 23922977725482944, -10577722854931902, 3525907618310634], [832272684833592926, 2622473368814272240, -150550199005148202, -12071880774171980, -12823003917482760, -2564600783496552], [206689896013797960, 783658575334757568, -183733509588563352, -5470489000346064, -14674133294853600, -2934826658970720], [-546271352177000836817, -2799447730144954042, 0, -11951409716193664, 8538062380029111, -2846020793343037]]
